@@ -32,6 +32,7 @@
 #include <ktar.h>
 #include <kzip.h>
 #include <kar.h>
+#include <k7zip.h>
 #include <kio/global.h>
 #include <kio_archive_debug.h>
 #include <kuser.h>
@@ -190,7 +191,7 @@ void ArchiveProtocolBase::createUDSEntry( const KArchiveEntry * archiveEntry, UD
     entry.fastInsert( KIO::UDSEntry::UDS_NAME, archiveEntry->name() );
     entry.fastInsert( KIO::UDSEntry::UDS_FILE_TYPE, archiveEntry->permissions() & S_IFMT ); // keep file type only
     entry.fastInsert( KIO::UDSEntry::UDS_SIZE, archiveEntry->isFile() ? ((KArchiveFile *)archiveEntry)->size() : 0L );
-    entry.fastInsert( KIO::UDSEntry::UDS_MODIFICATION_TIME, archiveEntry->date().toTime_t());
+    entry.fastInsert( KIO::UDSEntry::UDS_MODIFICATION_TIME, archiveEntry->date().toSecsSinceEpoch());
     entry.fastInsert( KIO::UDSEntry::UDS_ACCESS, archiveEntry->permissions() & 07777 ); // keep permissions only
     entry.fastInsert( KIO::UDSEntry::UDS_USER, archiveEntry->user());
     entry.fastInsert( KIO::UDSEntry::UDS_GROUP, archiveEntry->group());
@@ -328,7 +329,7 @@ void ArchiveProtocolBase::stat( const QUrl & url )
         if ( QT_STAT( QFile::encodeName( fullPath ), &buff ) == -1 )
         {
             // Should not happen, as the file was already stated by checkNewFile
-            error( KIO::ERR_COULD_NOT_STAT, url.toDisplayString() );
+            error( KIO::ERR_CANNOT_STAT, url.toDisplayString() );
             return;
         }
 
@@ -474,7 +475,7 @@ void ArchiveProtocolBase::get( const QUrl & url )
         if ( read != bufferSize )
         {
             qCWarning(KIO_ARCHIVE_LOG) << "Read" << read << "bytes but expected" << bufferSize ;
-            error( KIO::ERR_COULD_NOT_READ, url.toDisplayString() );
+            error( KIO::ERR_CANNOT_READ, url.toDisplayString() );
             delete io;
             return;
         }
