@@ -234,12 +234,12 @@ QUrl SMBSlave::checkURL(const QUrl &kurl_) const
     // NB: smbc has no way to resolve a name without also triggering auth etc.: we must
     //   rely on the system's ability to resolve DNSSD for this check.
     const QLatin1String wsdSuffix(".kio-discovery-wsd");
-    if (m_current_url.host().endsWith(wsdSuffix)) {
-        QString host = m_current_url.host();
+    if (kurl.host().endsWith(wsdSuffix)) {
+        QString host = kurl.host();
         host.chop(wsdSuffix.size());
         const QString dnssd(host + ".local");
-        auto dnssdHost = QHostInfo::fromName(dnssd);
-        if (dnssdHost.error() == QHostInfo::NoError) {
+        auto dnssdHost = KDNSSD::ServiceBrowser::resolveHostName(dnssd);
+        if (!dnssdHost.isNull()) {
             qCDebug(KIO_SMB_LOG) << "Resolved DNSSD name:" << dnssd;
             host = dnssd;
         } else {
@@ -357,7 +357,13 @@ SMBSlave::SMBError SMBSlave::errnumToKioError(const SMBUrl &url, const int errNu
                               "unsure about that - you can send it privately to the developers "
                               "if they ask for it)")};
     default:
-        return SMBError {ERR_INTERNAL, i18n("Unknown error condition in stat: %1", QString::fromLocal8Bit(strerror(errNum)))};
+        return SMBError {
+            // errnum is glued in to not break string freeze in 20.04
+            ERR_INTERNAL,
+                    i18n("Unknown error condition in stat: %1",
+                         QString("[%1] %2").arg(QString::number(errNum),
+                                                QString::fromLocal8Bit(strerror(errNum))))
+        };
     }
 }
 
