@@ -20,8 +20,9 @@
 #ifndef _THUMBNAIL_H_
 #define _THUMBNAIL_H_
 
-#include <QtCore/QHash>
-#include <QtGui/QPainter>
+#include <QHash>
+#include <QPainter>
+#include <QSet>
 
 #include <kio/slavebase.h>
 
@@ -32,9 +33,9 @@ class ThumbnailProtocol : public KIO::SlaveBase
 {
 public:
     ThumbnailProtocol(const QByteArray &pool, const QByteArray &app);
-    virtual ~ThumbnailProtocol();
+    ~ThumbnailProtocol() override;
 
-    virtual void get(const QUrl &url);
+    void get(const QUrl &url) override;
 
 protected:
     ThumbCreator* getThumbCreator(const QString& plugin);
